﻿using AutoMapper;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;
using Tsi1.DataLayer;
using Tsi1.DataLayer.Entities;

namespace Tsi1.BusinessLayer.Services
{
    public class ForumService : IForumService
    {
        private readonly Tsi1Context _context;

        private readonly IMapper _mapper;

        public ForumService(Tsi1Context context, IMapper mapper)
        {
            _context = context;
            _mapper = mapper;
        }

        public async Task<ServiceResult<Forum>> Create(ForumCreateDto newForum)
        {
            var result = new ServiceResult<Forum>();

            var forum = _mapper.Map<Forum>(newForum);

            _context.Forums.Add(forum);
            await _context.SaveChangesAsync();

            result.Data = forum;

            return result;
        }

        public async Task<ServiceResult<Forum>> Delete(int forumId)
        {
            var result = new ServiceResult<Forum>();

            var forum = await _context.Forums.FirstOrDefaultAsync(x => x.Id == forumId);

            if (forum == null)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.ForumDoesNotExist, forumId);
                return result;
            }

            _context.Forums.Remove(forum);

            await _context.SaveChangesAsync();

            return result;
        }

        public async Task<ServiceResult<List<ForumPreviewDto>>> GetForums(int courseId)
        {
            var result = new ServiceResult<List<ForumPreviewDto>>();

            var forums = await _context.Forums
                .Where(x => x.CourseId == courseId)
                .ToListAsync();

            var forumDtos = _mapper.Map<List<ForumPreviewDto>>(forums);

            result.Data = forumDtos;

            return result;
        }
    }
}
